#include <avr/pgmspace.h>

// Stores 128+127*sin(angle) for angles 0..63 (a full revolution is 256 "degrees").
const uint8_t sin_table[64] PROGMEM = {
	128, 131, 134, 137, 140, 144, 147, 150,
	153, 156, 159, 162, 165, 168, 171, 174,
	177, 179, 182, 185, 188, 191, 193, 196,
	199, 201, 204, 206, 209, 211, 213, 216,
	218, 220, 222, 224, 226, 228, 230, 232,
	234, 235, 237, 239, 240, 241, 243, 244,
	245, 246, 248, 249, 250, 250, 251, 252,
	253, 253, 254, 254, 254, 255, 255, 255,
};

// Compute 128+127*sin(angle) where angle is from 0..255.
uint8_t sin_uint8(uint8_t angle) {
	
	// Is the angle in the second (64..127) or fourth (192..255) quadrants?
	if (angle & 64) {
		// Mirror the angle to correct it, retaining the MSB.
		angle = (angle & 128) | (63 - (angle & 63));
	}
	
	// Fetch the plain angle value.
	uint8_t value = pgm_read_byte(sin_table + (angle & 63));
	
	// Is the angle in the third (128..191) or fourth (192..255) quadrants?
	if (angle & 128) {
		// Mirror the value to correct it.
		value = 255 - value;
	}
	
	return value;
}